package fr.username404.snowygui.gui.feature

import com.mojang.blaze3d.systems.RenderSystem
import com.mojang.blaze3d.vertex.PoseStack
import fr.username404.snowygui.ClickGui
import fr.username404.snowygui.config.Configuration
import fr.username404.snowygui.gui.ColoredElement
import fr.username404.snowygui.gui.FontUtil
import fr.username404.snowygui.gui.Renderable.Rendering.defaultRectFunc
import fr.username404.snowygui.gui.elements.ClickBox
import fr.username404.snowygui.gui.feature.ButtonInfo.Companion.Type
import org.jetbrains.annotations.ApiStatus
import kotlin.reflect.full.findAnnotation


@ApiStatus.Internal
abstract class ButtonImpl: ColoredElement(0.0, 0.0, 73, 8, opacity = 0.60F) {
    private val info = this::class.findAnnotation<ButtonInfo>() ?: throw Exception("Missing @Button annotaton")
    override var color: Colors = info.color
    val title = this@ButtonImpl::class.simpleName
    protected open fun execAction() = Unit
    private var wasWithinBounds: Boolean = false
    var toggled: Boolean = false; private set(value) {
        if (value) lightUp() else if (field) lightDown()
        if ((!(!field && !value))) {
            field = value
            execAction()
        } else field = value
    }
    private fun lightUp() { opacity += ButtonInfo.lightningFactor }
    private fun lightDown() { opacity -= ButtonInfo.lightningFactor }
    override fun mouseClicked(d: Double, e: Double, i: Int): Boolean {
        wasWithinBounds = withinBounds(d, e).also {
            if (it && (info.kind == Type.TOGGLE)) {
                toggled = !toggled
            }
        }
        return false
    }

    override fun mouseReleased(d: Double, e: Double, i: Int): Boolean {
        if (wasWithinBounds && (info.kind == Type.CLICK)) {
            toggled = true
        }; return false
    }
    override fun render(poseStack: PoseStack?) {
        RenderSystem.disableTexture()
        RenderSystem.enableBlend()
        defaultRectFunc()
        RenderSystem.enableTexture()
        RenderSystem.disableBlend()
        if (poseStack != null && title != null) {
            FontUtil.drawScaled(poseStack, title, x + 1, y + 1, 0.75F)
        }
    }
    init {
        if (info.kind == Type.TOGGLE) {
            Configuration.enabledFeatures[title]?.let {
                toggled = it
            }
        }
        ButtonInfo
        ClickGui.components.filterIsInstance<ClickBox>().find {
            it.name!!.key == info.parent.translationKey
        }?.buttons!!.add(this)
    }
}

